@@ -88,6 +88,7 @@ import Control.Lens.Setter
 import Control.Lens.Traversal
 import Control.Lens.Type
 import Data.Foldable
+import Data.Functor.Contravariant
 import Data.Functor.Identity
 import Data.Functor.Reverse
 import Data.Hashable
@@ -158,7 +159,7 @@ withIndex f = Indexed $ \i a -> snd <$> indexed f i (i, a)
 
 -- | When composed with an 'IndexedFold' or 'IndexedTraversal' this yields an
 -- ('Indexed') 'Fold' of the indices.
-asIndex :: (Indexable i p, Functor f, Gettable f) => Overloading' p (Indexed i) f s i
+asIndex :: (Indexable i p, Contravariant f, Functor f) => Overloading' p (Indexed i) f s i
 asIndex f = Indexed $ \i _ -> coerce (indexed f i i)
 {-# INLINE asIndex #-}
 
@@ -250,6 +251,7 @@ class Foldable f => FoldableWithIndex i f | f -> i where
   -- @
   ifoldr   :: (i -> a -> b -> b) -> b -> f a -> b
   ifoldr f z t = appEndo (ifoldMap (\i -> Endo #. f i) t) z
+  {-# INLINE ifoldr #-}
 
   -- | Left-associative fold of an indexed container with access to the index @i@.
   --
@@ -260,6 +262,7 @@ class Foldable f => FoldableWithIndex i f | f -> i where
   -- @
   ifoldl :: (i -> b -> a -> b) -> b -> f a -> b
   ifoldl f z t = appEndo (getDual (ifoldMap (\i -> Dual #. Endo #. flip (f i)) t)) z
+  {-# INLINE ifoldl #-}
 
   -- | /Strictly/ fold right over the elements of a structure with access to the index @i@.
   --
@@ -271,6 +274,7 @@ class Foldable f => FoldableWithIndex i f | f -> i where
   ifoldr' :: (i -> a -> b -> b) -> b -> f a -> b
   ifoldr' f z0 xs = ifoldl f' id xs z0
     where f' i k x z = k $! f i x z
+  {-# INLINE ifoldr' #-}
 
   -- | Fold over the elements of a structure with an index, associating to the left, but /strictly/.
   --
@@ -282,6 +286,7 @@ class Foldable f => FoldableWithIndex i f | f -> i where
   ifoldl' :: (i -> b -> a -> b) -> b -> f a -> b
   ifoldl' f z0 xs = ifoldr f' id xs z0
     where f' i x k z = k $! f i z x
+  {-# INLINE ifoldl' #-}
 
 -- | Obtain a 'Fold' by lifting an operation that returns a 'Foldable' result.
 --
@@ -451,7 +456,6 @@ class (FunctorWithIndex i t, FoldableWithIndex i t, Traversable t) => Traversabl
   itraversed = conjoined traverse (itraverse . indexed)
   {-# INLINE itraversed #-}
 
-
 -- | Traverse with an index (and the arguments flipped).
 --
 -- @
@@ -515,30 +519,39 @@ imapAccumL f s0 a = swap (Lazy.runState (itraverse (\i c -> Lazy.state (\s -> sw
 
 instance FunctorWithIndex i f => FunctorWithIndex i (Backwards f) where
   imap f  = Backwards . imap f . forwards
+  {-# INLINE imap #-}
 
 instance FoldableWithIndex i f => FoldableWithIndex i (Backwards f) where
   ifoldMap f = ifoldMap f . forwards
+  {-# INLINE ifoldMap #-}
 
 instance TraversableWithIndex i f => TraversableWithIndex i (Backwards f) where
   itraverse f = fmap Backwards . itraverse f . forwards
+  {-# INLINE itraverse #-}
 
 instance FunctorWithIndex i f => FunctorWithIndex i (Reverse f) where
   imap f = Reverse . imap f . getReverse
+  {-# INLINE imap #-}
 
 instance FoldableWithIndex i f => FoldableWithIndex i (Reverse f) where
   ifoldMap f = getDual . ifoldMap (\i -> Dual #. f i) . getReverse
+  {-# INLINE ifoldMap #-}
 
 instance TraversableWithIndex i f => TraversableWithIndex i (Reverse f) where
   itraverse f = fmap Reverse . forwards . itraverse (\i -> Backwards . f i) . getReverse
+  {-# INLINE itraverse #-}
 
 instance FunctorWithIndex () Identity where
   imap f (Identity a) = Identity (f () a)
+  {-# INLINE imap #-}
 
 instance FoldableWithIndex () Identity where
   ifoldMap f (Identity a) = f () a
+  {-# INLINE ifoldMap #-}
 
 instance TraversableWithIndex () Identity where
   itraverse f (Identity a) = Identity <$> f () a
+  {-# INLINE itraverse #-}
 
 instance FunctorWithIndex k ((,) k) where
   imap f (k,a) = (k, f k a)
@@ -563,6 +576,16 @@ instance TraversableWithIndex Int [] where
   itraverse = itraverseOf traversed
   {-# INLINE itraverse #-}
 
+instance FunctorWithIndex () Maybe where
+  imap f = fmap (f ())
+  {-# INLINE imap #-}
+instance FoldableWithIndex () Maybe where
+  ifoldMap f = foldMap (f ())
+  {-# INLINE ifoldMap #-}
+instance TraversableWithIndex () Maybe where
+  itraverse f = traverse (f ())
+  {-# INLINE itraverse #-}
+
 -- | The position in the 'Seq' is available as the index.
 instance FunctorWithIndex Int Seq where
   imap = iover itraversed
@@ -581,9 +604,13 @@ instance FoldableWithIndex Int Vector where
   ifoldMap = ifoldMapOf itraversed
   {-# INLINE ifoldMap #-}
   ifoldr = V.ifoldr
+  {-# INLINE ifoldr #-}
   ifoldl = V.ifoldl . flip
+  {-# INLINE ifoldl #-}
   ifoldr' = V.ifoldr'
+  {-# INLINE ifoldr' #-}
   ifoldl' = V.ifoldl' . flip
+  {-# INLINE ifoldl' #-}
 instance TraversableWithIndex Int Vector where
   itraverse f = sequenceA . V.imap f
   {-# INLINE itraverse #-}
