@@ -104,7 +104,7 @@ import           Data.Tree
 -- @
 -- import Control.Applicative
 -- import Control.Lens
--- import Control.Plated
+-- import Control.Lens.Plated
 -- import Data.Data
 -- import Data.Data.Lens ('Data.Data.Lens.uniplate')
 -- @
@@ -136,7 +136,7 @@ import           Data.Tree
 -- @
 -- import Control.Applicative
 -- import Control.Lens
--- import Control.Plated
+-- import Control.Lens.Plated
 -- import Data.Data
 -- import Data.Data.Lens ('Data.Data.Lens.uniplate')
 -- @
@@ -330,13 +330,13 @@ universe = universeOf plate
 -- @
 -- 'universeOf' :: 'Fold' a a -> a -> [a]
 -- @
-universeOf :: Getting [a] a b a b -> a -> [a]
+universeOf :: Getting [a] a a -> a -> [a]
 universeOf l = go where
   go a = a : foldMapOf l go a
 {-# INLINE universeOf #-}
 
 -- | Given a 'Fold' that knows how to find 'Plated' parts of a container retrieve them and all of their descendants, recursively.
-universeOn ::  Plated a => Getting [a] s t a a -> s -> [a]
+universeOn ::  Plated a => Getting [a] s a -> s -> [a]
 universeOn b = universeOnOf b plate
 {-# INLINE universeOn #-}
 
@@ -346,7 +346,7 @@ universeOn b = universeOnOf b plate
 -- @
 -- 'toListOf' l ≡ 'universeOnOf' l 'ignored'
 -- @
-universeOnOf :: Getting [a] s t a b -> Getting [a] a b a b -> s -> [a]
+universeOnOf :: Getting [a] s a -> Getting [a] a a -> s -> [a]
 universeOnOf b = foldMapOf b . universeOf
 {-# INLINE universeOnOf #-}
 
@@ -429,7 +429,7 @@ transformMOf l f = go where
 -- @
 -- 'transformMOnOf' :: 'Monad' m => 'Traversal'' s a -> 'Traversal'' a a -> (a -> m a) -> s -> m s
 -- @
-transformMOnOf :: Monad m => LensLike (WrappedMonad m) s a a a -> LensLike' (WrappedMonad m) a a -> (a -> m a) -> s -> m a
+transformMOnOf :: Monad m => LensLike (WrappedMonad m) s t a a -> LensLike' (WrappedMonad m) a a -> (a -> m a) -> s -> m t
 transformMOnOf b l = mapMOf b . transformMOf l
 {-# INLINE transformMOnOf #-}
 
@@ -459,7 +459,7 @@ contexts = contextsOf plate
 -- @
 --
 -- @
--- 'contextsOf' :: 'Traversal'' a a -> a -> ['Context' a a]
+-- 'contextsOf' :: 'Traversal'' a a -> a -> ['Context' a a a]
 -- @
 contextsOf :: ATraversal' a a -> a -> [Context a a a]
 contextsOf l x = sell x : f (map context (holesOf l x)) where
@@ -558,7 +558,7 @@ holesOnOf b l = holesOf (b . l)
 -- @
 -- 'paraOf' :: 'Fold' a a -> (a -> [r] -> r) -> a -> r
 -- @
-paraOf :: Getting (Endo [a]) a b a b -> (a -> [r] -> r) -> a -> r
+paraOf :: Getting (Endo [a]) a a -> (a -> [r] -> r) -> a -> r
 paraOf l f = go where
   go a = f a (go <$> toListOf l a)
 {-# INLINE paraOf #-}
