@@ -25,8 +25,9 @@ module Control.Lens.Review
   , review, reviews
   , reuse, reuses
   , ( # )
-  -- * Reviewable Profunctors
-  , Reviewable(..)
+  , Bifunctor(bimap)
+  , retagged
+  , Reviewable
   ) where
 
 import Control.Monad.Reader as Reader
@@ -35,9 +36,11 @@ import Control.Lens.Getter
 import Control.Lens.Internal.Review
 import Control.Lens.Internal.Setter
 import Control.Lens.Type
+import Data.Bifunctor
 import Data.Functor.Identity
 import Data.Profunctor
 import Data.Profunctor.Unsafe
+import Data.Void
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
@@ -46,7 +49,7 @@ import Data.Profunctor.Unsafe
 -- >>> let isLeft  (Left  _) = True; isLeft  _ = False
 -- >>> let isRight (Right _) = True; isRight _ = False
 
-infixr 9 #
+infixr 8 #
 
 ------------------------------------------------------------------------------
 -- Review
@@ -58,7 +61,7 @@ infixr 9 #
 --
 -- You can generate a 'Review' by using 'unto'. You can also use any 'Prism' or 'Iso'
 -- directly as a 'Review'.
-type Review s t a b = forall p f. (Reviewable p, Settable f) => Overloaded p f s t a b
+type Review s t a b = forall p f. (Profunctor p, Bifunctor p, Settable f) => Overloaded p f s t a b
 
 -- | A 'Simple' 'Review'
 type Review' t b = Review t t b b
@@ -75,8 +78,9 @@ type AReview' t b = AReview t t b b
 -- @
 -- 'unto' :: (b -> t) -> 'Review'' t b
 -- @
-unto :: (Reviewable p, Functor f) => (b -> t) -> Overloaded p f s t a b
-unto f = retagged . rmap (fmap f)
+unto :: (Profunctor p, Bifunctor p, Functor f) => (b -> t) -> Overloaded p f s t a b
+unto f = first absurd . lmap absurd . rmap (fmap f)
+{-# INLINE unto #-}
 
 -- | Turn a 'Prism' or 'Control.Lens.Iso.Iso' around to build a 'Getter'.
 --
