@@ -59,6 +59,7 @@ module Control.Lens.Traversal
   , transposeOf
   , mapAccumLOf, mapAccumROf
   , scanr1Of, scanl1Of
+  , failover
 
   -- * Monomorphic Traversals
   , cloneTraversal
@@ -107,16 +108,19 @@ module Control.Lens.Traversal
   , Bazaar(..)
   , Bazaar'
   , loci
+  , iloci
   ) where
 
 import Control.Applicative as Applicative
 import Control.Applicative.Backwards
 import Control.Category
 import Control.Comonad
+import Control.Monad
 import Control.Lens.Combinators
 import Control.Lens.Fold
-import Control.Lens.Internal.Context
+import Control.Lens.Getter (coerced)
 import Control.Lens.Internal.Bazaar
+import Control.Lens.Internal.Context
 import Control.Lens.Internal.Indexed
 import Control.Lens.Type
 import Control.Monad.Trans.State.Lazy
@@ -124,6 +128,7 @@ import Data.Functor.Compose
 import Data.Int
 import Data.IntMap as IntMap
 import Data.Map as Map
+import Data.Monoid
 import Data.Traversable
 import Data.Tuple (swap)
 import Data.Profunctor
@@ -137,6 +142,7 @@ import Prelude hiding ((.),id)
 -- >>> import Control.DeepSeq (NFData (..), force)
 -- >>> import Control.Exception (evaluate)
 -- >>> import Data.Maybe (fromMaybe)
+-- >>> import Data.Void
 -- >>> import System.Timeout (timeout)
 -- >>> let timingOut :: NFData a => a -> IO a; timingOut = fmap (fromMaybe (error "timeout")) . timeout (5*10^6) . evaluate . force
 
@@ -208,9 +214,9 @@ type Traversing' p f s a = Traversing p f s s a a
 -- @
 --
 -- @
--- 'traverseOf' :: 'Iso' s t a b       -> (a -> f b) -> s -> f t
--- 'traverseOf' :: 'Lens' s t a b      -> (a -> f b) -> s -> f t
--- 'traverseOf' :: 'Traversal' s t a b -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Functor' f => 'Iso' s t a b       -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Functor' f => 'Lens' s t a b      -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Applicative' f => 'Traversal' s t a b -> (a -> f b) -> s -> f t
 -- @
 traverseOf :: Over p f s t a b -> p a (f b) -> s -> f t
 traverseOf = id
@@ -237,9 +243,9 @@ traverseOf = id
 -- @
 --
 -- @
--- 'forOf' :: 'Iso' s t a b -> s -> (a -> f b) -> f t
--- 'forOf' :: 'Lens' s t a b -> s -> (a -> f b) -> f t
--- 'forOf' :: 'Traversal' s t a b -> s -> (a -> f b) -> f t
+-- 'forOf' :: 'Functor' f => 'Iso' s t a b -> s -> (a -> f b) -> f t
+-- 'forOf' :: 'Functor' f => 'Lens' s t a b -> s -> (a -> f b) -> f t
+-- 'forOf' :: 'Applicative' f => 'Traversal' s t a b -> s -> (a -> f b) -> f t
 -- @
 forOf :: Over p f s t a b -> s -> p a (f b) -> f t
 forOf = flip
@@ -257,8 +263,8 @@ forOf = flip
 -- @
 --
 -- @
--- 'sequenceAOf' ::                  'Iso' s t (f b) b       -> s -> f t
--- 'sequenceAOf' ::                  'Lens' s t (f b) b      -> s -> f t
+-- 'sequenceAOf' :: 'Functor' f => 'Iso' s t (f b) b       -> s -> f t
+-- 'sequenceAOf' :: 'Functor' f => 'Lens' s t (f b) b      -> s -> f t
 -- 'sequenceAOf' :: 'Applicative' f => 'Traversal' s t (f b) b -> s -> f t
 -- @
 sequenceAOf :: LensLike f s t (f b) b -> s -> f t
@@ -277,8 +283,8 @@ sequenceAOf l = l id
 -- @
 --
 -- @
--- 'mapMOf' ::            'Iso' s t a b       -> (a -> m b) -> s -> m t
--- 'mapMOf' ::            'Lens' s t a b      -> (a -> m b) -> s -> m t
+-- 'mapMOf' :: 'Monad' m => 'Iso' s t a b       -> (a -> m b) -> s -> m t
+-- 'mapMOf' :: 'Monad' m => 'Lens' s t a b      -> (a -> m b) -> s -> m t
 -- 'mapMOf' :: 'Monad' m => 'Traversal' s t a b -> (a -> m b) -> s -> m t
 -- @
 mapMOf :: Profunctor p => Over p (WrappedMonad m) s t a b -> p a (m b) -> s -> m t
@@ -297,8 +303,8 @@ mapMOf l cmd = unwrapMonad #. l (WrapMonad #. cmd)
 -- @
 --
 -- @
--- 'forMOf' ::            'Iso' s t a b       -> s -> (a -> m b) -> m t
--- 'forMOf' ::            'Lens' s t a b      -> s -> (a -> m b) -> m t
+-- 'forMOf' :: 'Monad' m => 'Iso' s t a b       -> s -> (a -> m b) -> m t
+-- 'forMOf' :: 'Monad' m => 'Lens' s t a b      -> s -> (a -> m b) -> m t
 -- 'forMOf' :: 'Monad' m => 'Traversal' s t a b -> s -> (a -> m b) -> m t
 -- @
 forMOf :: Profunctor p => Over p (WrappedMonad m) s t a b -> s -> p a (m b) -> m t
@@ -317,8 +323,8 @@ forMOf l a cmd = unwrapMonad (l (WrapMonad #. cmd) a)
 -- @
 --
 -- @
--- 'sequenceOf' ::            'Iso' s t (m b) b       -> s -> m t
--- 'sequenceOf' ::            'Lens' s t (m b) b      -> s -> m t
+-- 'sequenceOf' :: 'Monad' m => 'Iso' s t (m b) b       -> s -> m t
+-- 'sequenceOf' :: 'Monad' m => 'Lens' s t (m b) b      -> s -> m t
 -- 'sequenceOf' :: 'Monad' m => 'Traversal' s t (m b) b -> s -> m t
 -- @
 sequenceOf :: LensLike (WrappedMonad m) s t (m b) b -> s -> m t
@@ -431,6 +437,12 @@ loci :: Traversal (Bazaar (->) a c s) (Bazaar (->) b c s) a b
 loci f w = getCompose (runBazaar w (Compose #. fmap sell . f))
 {-# INLINE loci #-}
 
+-- | This 'IndexedTraversal' allows you to 'traverse' the individual stores in
+-- a 'Bazaar' with access to their indices.
+iloci :: IndexedTraversal i (Bazaar (Indexed i) a c s) (Bazaar (Indexed i) b c s) a b
+iloci f w = getCompose (runBazaar w (Compose #. Indexed (\i -> fmap (indexed sell i) . indexed f i)))
+{-# INLINE iloci #-}
+
 -------------------------------------------------------------------------------
 -- Parts and Holes
 -------------------------------------------------------------------------------
@@ -592,7 +604,7 @@ unsafeSingular l pafb s = case pins b of
 ------------------------------------------------------------------------------
 
 ins :: Bizarre (->) w => w a b t -> [a]
-ins = toListOf bazaar
+ins = toListOf (coerced bazaar)
 {-# INLINE ins #-}
 
 pins :: (Bizarre p w, Corepresentable p) => w a b t -> [Corep p a]
@@ -641,7 +653,7 @@ both f ~(a,a') = (,) <$> f a <*> f a'
 -- 'beside' :: 'Traversal' s t a b                -> 'Traversal' s' t' a b                -> 'Traversal' (s,s') (t,t') a b
 -- 'beside' :: 'Lens' s t a b                     -> 'Lens' s' t' a b                     -> 'Traversal' (s,s') (t,t') a b
 -- 'beside' :: 'Fold' s a                         -> 'Fold' s' a                          -> 'Fold' (s,s') a
--- 'beside' :: 'Getter' s a                       -> 'Getter' s' a                        -> 'Traversal' (s,s') a
+-- 'beside' :: 'Getter' s a                       -> 'Getter' s' a                        -> 'Fold' (s,s') a
 -- 'beside' :: 'Action' m s a                     -> 'Action' m s' a                      -> 'MonadicFold' m (s,s') a
 -- 'beside' :: 'MonadicFold' m s a                -> 'MonadicFold' m s' a                 -> 'MonadicFold' m (s,s') a
 -- @
@@ -650,7 +662,7 @@ both f ~(a,a') = (,) <$> f a <*> f a'
 -- 'beside' :: 'IndexedTraversal' i s t a b       -> 'IndexedTraversal' i s' t' a b       -> 'IndexedTraversal' i (s,s') (t,t') a b
 -- 'beside' :: 'IndexedLens' i s t a b            -> 'IndexedLens' i s' t' a b            -> 'IndexedTraversal' i (s,s') (t,t') a b
 -- 'beside' :: 'IndexedFold' i s a                -> 'IndexedFold' i s' a                 -> 'IndexedFold' i (s,s') a
--- 'beside' :: 'IndexedGetter' i s a              -> 'IndexedGetter' i s' a               -> 'IndexedTraversal' i (s,s') a
+-- 'beside' :: 'IndexedGetter' i s a              -> 'IndexedGetter' i s' a               -> 'IndexedFold' i (s,s') a
 -- 'beside' :: 'IndexedAction' i m s a            -> 'IndexedAction' i m s' a             -> 'IndexedMonadicFold' i m (s,s') a
 -- 'beside' :: 'IndexedMonadicFold' i m s a       -> 'IndexedMonadicFold' i m s' a        -> 'IndexedMonadicFold' i m (s,s') a
 -- @
@@ -659,7 +671,7 @@ both f ~(a,a') = (,) <$> f a <*> f a'
 -- 'beside' :: 'IndexPreservingTraversal' s t a b -> 'IndexPreservingTraversal' s' t' a b -> 'IndexPreservingTraversal' (s,s') (t,t') a b
 -- 'beside' :: 'IndexPreservingLens' s t a b      -> 'IndexPreservingLens' s' t' a b      -> 'IndexPreservingTraversal' (s,s') (t,t') a b
 -- 'beside' :: 'IndexPreservingFold' s a          -> 'IndexPreservingFold' s' a           -> 'IndexPreservingFold' (s,s') a
--- 'beside' :: 'IndexPreservingGetter' s a        -> 'IndexPreservingGetter' s' a         -> 'IndexPreservingTraversal' (s,s') a
+-- 'beside' :: 'IndexPreservingGetter' s a        -> 'IndexPreservingGetter' s' a         -> 'IndexPreservingFold' (s,s') a
 -- 'beside' :: 'IndexPreservingAction' m s a      -> 'IndexPreservingAction' m s' a       -> 'IndexPreservingMonadicFold' m (s,s') a
 -- 'beside' :: 'IndexPreservingMonadicFold' m s a -> 'IndexPreservingMonadicFold' m s' a  -> 'IndexPreservingMonadicFold' m (s,s') a
 -- @
@@ -756,12 +768,12 @@ dropping n l pafb s = snd $ runIndexing (l paifb s) 0 where
 -- former can execute at full speed, while the latter needs to round trip through
 -- the 'Bazaar'.
 --
--- >>> let foo l a = (view (cloneTraversal l) a, set (cloneTraversal l) 10 a)
+-- >>> let foo l a = (view (coerced (cloneTraversal l)) a, set (cloneTraversal l) 10 a)
 -- >>> foo both ("hello","world")
 -- ("helloworld",(10,10))
 --
 -- @
--- 'cloneTraversal' :: 'LensLike' ('Bazaar' a b) s t a b -> 'Traversal' s t a b
+-- 'cloneTraversal' :: 'LensLike' ('Bazaar' (->) a b) s t a b -> 'Traversal' s t a b
 -- @
 cloneTraversal :: ATraversal s t a b -> Traversal s t a b
 cloneTraversal l f = bazaar f . l sell
@@ -793,8 +805,8 @@ cloneIndexedTraversal l f = bazaar (Indexed (indexed f)) . l sell
 -- @
 --
 -- @
--- 'itraverseOf' :: 'IndexedLens' i s t a b      -> (i -> a -> f b) -> s -> f t
--- 'itraverseOf' :: 'IndexedTraversal' i s t a b -> (i -> a -> f b) -> s -> f t
+-- 'itraverseOf' :: 'Functor' f     => 'IndexedLens' i s t a b      -> (i -> a -> f b) -> s -> f t
+-- 'itraverseOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b -> (i -> a -> f b) -> s -> f t
 -- @
 itraverseOf :: (Indexed i a (f b) -> s -> f t) -> (i -> a -> f b) -> s -> f t
 itraverseOf l = l .# Indexed
@@ -808,8 +820,8 @@ itraverseOf l = l .# Indexed
 -- @
 --
 -- @
--- 'iforOf' :: 'IndexedLens' i s t a b      -> s -> (i -> a -> f b) -> f t
--- 'iforOf' :: 'IndexedTraversal' i s t a b -> s -> (i -> a -> f b) -> f t
+-- 'iforOf' :: 'Functor' f => 'IndexedLens' i s t a b      -> s -> (i -> a -> f b) -> f t
+-- 'iforOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b -> s -> (i -> a -> f b) -> f t
 -- @
 iforOf :: (Indexed i a (f b) -> s -> f t) -> s -> (i -> a -> f b) -> f t
 iforOf = flip . itraverseOf
@@ -905,6 +917,9 @@ traversed64 = conjoined traverse (indexing64 traverse)
 -- @
 -- 'ignored' ≡ 'const' 'pure'
 -- @
+--
+-- >>> 6 & ignored %~ absurd
+-- 6
 ignored :: Applicative f => pafb -> s -> f s
 ignored _ = pure
 {-# INLINE ignored #-}
@@ -1007,3 +1022,19 @@ elementsOf l p iafb s = snd $ runIndexing (l (\a -> Indexing (\i -> i `seq` (i +
 elements :: Traversable t => (Int -> Bool) -> IndexedTraversal' Int (t a) a
 elements = elementsOf traverse
 {-# INLINE elements #-}
+
+-- | Try to map a function over this 'Traversal', failing if the 'Traversal' has no targets.
+--
+-- >>> failover (element 3) (*2) [1,2] :: Maybe [Int]
+-- Nothing
+--
+-- >>> failover _Left (*2) (Right 4) :: Maybe (Either Int Int)
+-- Nothing
+--
+-- >>> failover _Right (*2) (Right 4) :: Maybe (Either Int Int)
+-- Just (Right 8)
+failover :: MonadPlus m => LensLike ((,) Any) s t a b -> (a -> b) -> s -> m t
+failover l f s = case l ((,) (Any True) . f) s of
+  (Any True, t)  -> return t
+  (Any False, _) -> mzero
+{-# INLINE failover #-}
