@@ -21,6 +21,6 @@ import Data.Monoid
 -- 'Getter' or 'Fold' according to the given strategy.
 --
 -- @'seqFoldable' = 'seqOf' 'folded'@
-seqOf :: Getting (Endo [a]) s t a b -> Strategy a -> Strategy s
+seqOf :: Getting (Endo [a]) s a -> Strategy a -> Strategy s
 seqOf l s = seqList s . toListOf l
 {-# INLINE seqOf #-}
