@@ -2,7 +2,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Tree.Lens
--- Copyright   :  (C) 2012 Edward Kmett
+-- Copyright   :  (C) 2012-13 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -12,21 +12,24 @@
 
 module Data.Tree.Lens
   ( root
-  , children
+  , branches
   ) where
 
 import Control.Lens
 import Data.Functor
-import Data.List.Lens
 import Data.Tree
 
 -- | A 'Lens' that focuses on the root of a 'Tree'.
-root :: Simple Lens (Tree a) a
+--
+-- >>> view root $ Node 42 []
+-- 42
+root :: Lens' (Tree a) a
 root f (Node a as) = (`Node` as) <$> f a
 {-# INLINE root #-}
 
--- | A 'Traversal' of the direct descendants of the root of a 'Tree'
--- indexed by its position in the list of children
-children :: SimpleIndexedTraversal Int (Tree a) (Tree a)
-children = index $ \ f (Node a as) -> Node a <$> withIndex traverseList f as
-{-# INLINE children #-}
+-- | A 'Lens' returning the direct descendants of the root of a 'Tree'
+--
+-- @'view' 'branches' ≡ 'subForest'@
+branches :: Lens' (Tree a) [Tree a]
+branches f (Node a as) = Node a <$> f as
+{-# INLINE branches #-}
