@@ -24,7 +24,6 @@ module Data.Vector.Generic.Lens
   , asStream
   , asStreamR
   , cloned
-  , reversed
   -- * Lenses
   , sliced
   -- * Traversal of individual indices
@@ -68,7 +67,7 @@ sliced i n f v = f (slice i n v) <&> \ v0 -> v // zip [i..i+n-1] (V.toList v0)
 --
 -- >>> toVectorOf both (8,15) :: Vector.Vector Int
 -- fromList [8,15]
-toVectorOf :: Vector v a => Getting (Endo [a]) s t a b -> s -> v a
+toVectorOf :: Vector v a => Getting (Endo [a]) s a -> s -> v a
 toVectorOf l s = fromList (toListOf l s)
 {-# INLINE toVectorOf #-}
 
@@ -103,18 +102,9 @@ cloned = iso clone new
 -- | Convert a 'Vector' to a version that doesn't retain any extra
 -- memory.
 forced :: Vector v a => Iso' (v a) (v a)
-forced = iso force force
+forced = involuted force
 {-# INLINE forced #-}
 
--- | Convert a 'Vector' to a version with all the elements in the
--- reverse order.
---
--- >>> Vector.fromList [1,2,3] ^. reversed
--- fromList [3,2,1]
-reversed :: Vector v a => Iso' (v a) (v a)
-reversed = iso reverse reverse
-{-# INLINE reversed #-}
-
 -- | This 'Traversal' will ignore any duplicates in the supplied list
 -- of indices.
 --
