@@ -21,7 +21,6 @@ module Data.Vector.Lens
   ( toVectorOf
   -- * Isomorphisms
   , vector
-  , reversed
   , forced
   -- * Lenses
   , sliced
@@ -60,7 +59,7 @@ sliced i n f v = f (slice i n v) <&> \ v0 -> v // zip [i..i+n-1] (toList v0)
 --
 -- >>> toVectorOf both (8,15)
 -- fromList [8,15]
-toVectorOf :: Getting (Endo [a]) s t a b -> s -> Vector a
+toVectorOf :: Getting (Endo [a]) s a -> s -> Vector a
 toVectorOf l s = fromList (toListOf l s)
 {-# INLINE toVectorOf #-}
 
@@ -78,15 +77,6 @@ vector :: Iso [a] [b] (Vector a) (Vector b)
 vector = iso fromList toList
 {-# INLINE vector #-}
 
--- | Convert a 'Vector' to a version with all the elements in the
--- reverse order.
---
--- >>> Vector.fromList [1,2,3] ^. reversed
--- fromList [3,2,1]
-reversed :: Iso (Vector a) (Vector b) (Vector a) (Vector b)
-reversed = iso reverse reverse
-{-# INLINE reversed #-}
-
 -- | Convert a 'Vector' to a version that doesn't retain any extra
 -- memory.
 forced :: Iso (Vector a) (Vector b) (Vector a) (Vector b)
